/*
 * Copyright (c) 2004  Brian S. Dean <bsd@bdmicro.com>
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY BRIAN S. DEAN ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL BRIAN S. DEAN BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 */

/* $Id: uart.h,v 1.4 2005/03/22 19:12:14 bsd Exp $ */

#ifndef __uart_h__
#define __uart_h__

#include <inttypes.h>

#include "ringbuf.h"

#define DATA9 0

#define DEF_UART 0
#define AUX_UART 1


#define UART0_BAUD 9600L

//#define UART1_BAUD 9600L
#define UART1_BAUD 115200L


#define DEF_UART_BAUD UART0_BAUD
#define AUX_UART_BAUD UART1_BAUD


extern volatile RINGBUF8 uartbuf;
extern volatile RINGBUF8 uart1buf;
extern volatile uint8_t uart0_rx;
extern volatile uint8_t uart1_rx;


#if AUX_UART == 1
static __inline__ void uart_enable_9th(void)
{
  UCSR1B |= _BV(TXB8);
}

static __inline__ void uart_disable_9th(void)
{
  UCSR1B &= ~_BV(TXB8);
}
#else
static __inline__ void uart_enable_9th(void)
{
  UCSR0B |= _BV(TXB8);
}

static __inline__ void uart_disable_9th(void)
{
  UCSR0B &= ~_BV(TXB8);
}
#endif


void uart_putc(uint8_t uart, char c);

void uarts_init(uint16_t uart0_brr, uint16_t uart1_brr);

void uart_tx_drain(uint8_t uart);

#endif

